"""
SPDX-License-Identifier: GPL-3.0-or-later

Retrieve configuration for use with the rest of execution.

Read the ncmonitor(5) manpage for an overview of configuration.
"""

from logging import getLogger
import re

from yaml import safe_load

DEFAULT_CONFIG_PATH = '/etc/ncmonitor/ncmonitor.conf'
VALID_HOSTNAME_RE = r'^([a-z\d](-*[a-z\d])*)(\.([a-z\d](-*[a-z\d])*))*$'

logger = getLogger(__name__)


class NCConfigError(Exception):
    """
    Base exception class.
    """


def is_valid_hostname(hostname: str) -> bool:
    """
    Rough validatation of the configured domain name.
    """
    logger.debug("Validating hostname %s", hostname)
    compiled = re.compile(VALID_HOSTNAME_RE, re.IGNORECASE)
    logger.debug("Does %s match regex? %s", hostname, compiled.match(hostname))
    return bool(compiled.match(hostname))


# pylint: disable-next=too-few-public-methods
class NCMonitorConfig:
    """
    Represents a configuration for ncmonitor
    """

    def __init__(self, path: str = DEFAULT_CONFIG_PATH) -> None:
        self.config_file = {}
        self.path = path
        self.nameservers = []
        self.mark_user = None
        self.mark_pass = None

        self._load_config(self.path)

    def _load_config(self, load_path: str) -> None:
        """
        Load a configuration file from the specified filename.
        """
        logger.debug("Loading configuration file %s.", load_path)
        with open(load_path, encoding='utf-8') as conf_file:
            self.config_file = safe_load(conf_file)

        self.nameservers.clear()
        try:
            for host in self.config_file["nameservers"]:
                if is_valid_hostname(host):
                    self.nameservers.append(host)
                    logger.debug("Adding %s to list of domains", host)
                else:
                    logger.exception("%s is not a valid hostname.", host)
                    raise NCConfigError
        except (KeyError, TypeError) as exc:
            logger.exception("Invalid config file; Have you specified any nameservers?")
            raise NCConfigError from exc

        try:
            self.mark_user = self.config_file["markmonitor"]["username"]
            self.mark_pass = self.config_file["markmonitor"]["password"]
        except (KeyError, TypeError) as exc:
            logger.exception("Missing credentials for MarkMonitor")
            raise NCConfigError from exc
