"""
SPDX-License-Identifier: GPL-3.0-or-later
"""

import tempfile
from pathlib import Path

import yaml
from git import Repo
from git.exc import GitError

from .domain import Domain

NCREDIR_ZONE = 'ncredir-parking'
DNS_REPO = 'https://gerrit-replica.wikimedia.org/r/operations/dns'
PUPPET_REPO = 'https://gerrit-replica.wikimedia.org/r/operations/puppet'

REDIRECT_COMMANDS = ('funnel', 'rewrite', 'override')


class GitRepositoryError(Exception):
    """
    Base exception class.
    """


class GitRepository:
    """
    Parent class: Clone a Git repository locally.
    """
    def __init__(self, url, depth=1):
        self.url = url
        self.depth = depth
        self.temp_directory = None

    def __enter__(self):
        self.temp_directory = tempfile.TemporaryDirectory()
        try:
            Repo.clone_from(self.url, self.temp_directory.name, depth=self.depth)
        except (IOError, GitError) as clone_error:
            raise GitRepositoryError from clone_error

        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.temp_directory:
            self.temp_directory.cleanup()


# pylint: disable-next=too-few-public-methods
class DNSRepository(GitRepository):
    """
    Clone the DNS repository locally and parse the domains listed.
    """
    def __init__(self, url=DNS_REPO, depth=1):
        super().__init__(url, depth)

    def get_domain_list(self, nczone=NCREDIR_ZONE):
        """
        Parse through ncredir's listed zones in the DNS repository and yield
        values back.
        """
        templates_path = Path(self.temp_directory.name).joinpath('templates')
        if not templates_path.is_dir():
            raise GitRepositoryError("Templates directory doesn't exist")

        zone_path = templates_path.joinpath(nczone).resolve()

        if not zone_path.is_file():
            raise GitRepositoryError("ncredir zone doesn't exist")

        for p in templates_path.iterdir():
            if not p.is_symlink():
                continue

            if p.resolve().samefile(zone_path):
                yield Domain(p.name)


# pylint: disable-next=too-few-public-methods
class ACMEChiefConfig(GitRepository):
    """
    Clone the Puppet repository and parse acme-chief's configuration to extract
    (and then yield) domain names and associated nameservers.
    """
    def __init__(self, url=PUPPET_REPO, depth=1):
        super().__init__(url, depth)

    def get_domain_list(self, acme_chief_config_path='hieradata/common.yaml'):
        """
        Parse Puppet's hieradata for acme-chief certificate domains.
        """
        config_path = Path(self.temp_directory.name).joinpath(acme_chief_config_path)
        domain_list = set()
        with open(config_path, encoding="utf-8") as config_file:
            try:
                config = yaml.safe_load(config_file.read())
                if 'shared_acme_certificates' not in config:
                    raise GitRepositoryError('Invalid acme-chief config')
                for cert_name, cert_config in config['shared_acme_certificates'].items():
                    if 'non-canonical-redirect-' not in cert_name:
                        continue
                    for sni in cert_config['SNI']:
                        domain_list.add(sni.lstrip('*.'))
            except (IOError, yaml.error.YAMLError) as yaml_error:
                raise GitRepositoryError('Invalid acme-chief config') from yaml_error

        for domain_name in domain_list:
            yield Domain(domain_name)


# pylint: disable-next=too-few-public-methods
class NCRedirRedirects(GitRepository):
    """
    Clone the Puppet repository locally and parse through ncredir's list of
    domains to redirect.
    """
    def __init__(self, url=PUPPET_REPO, depth=1):
        super().__init__(url, depth)

    def get_domain_list(self, nc_redirects_path='modules/ncredir/files/nc_redirects.dat'):
        """
        Parse through ncredir's redirects file and extract/yield the domains
        and their associated nameservers.
        """
        config_path = Path(self.temp_directory.name).joinpath(nc_redirects_path)
        domain_list = set()
        with open(config_path, encoding="utf-8") as config_file:
            try:
                config = config_file.read()
                for config_line in config.splitlines():
                    line = config_line.strip()
                    if len(line) == 0 or line.startswith('#'):
                        continue
                    fields = line.split('\t')
                    if fields[0] not in REDIRECT_COMMANDS:
                        continue
                    domain_list.add(fields[1].lstrip('*').lstrip('.'))
            except IOError as config_error:
                raise GitRepositoryError('Invalid nc_redirects.dat') from config_error

            for domain_name in domain_list:
                yield Domain(domain_name)
