"""
SPDX-License-Identifier: GPL-3.0-or-later
"""

from argparse import ArgumentParser
from collections import defaultdict
import os
from pathlib import Path
from typing import Optional

from .config import NCMonitorConfig
from .gitrepo import DNSRepository, ACMEChiefConfig, NCRedirRedirects
from .markmonitor import MarkMonitor


def get_conf_file(user_path: Optional[str] = None) -> str:
    """
    Locate user-supplied configuration file or check in pre-configured paths.
    """
    conf_name = "ncmonitor.conf"
    default_paths = [
        Path(Path.cwd() / conf_name),
        Path(os.environ.get("XDG_CONFIG_HOME", Path.home() / ".config" / "ncmonitor" / conf_name)),
        Path("/etc/ncmonitor" / conf_name),
    ]
    if user_path is None:
        for candidate in default_paths:
            if isinstance(candidate, Path) and candidate.is_file():
                conf_file = candidate
                break
    if conf_file is None:
        raise FileNotFoundError

    return conf_file


def main() -> int:
    """
    Get acme-chief, ncredir, and MarkMonitor domain information. Compare and
    report on any inconsistencies.
    """
    parser = ArgumentParser(
        description="Look up registrar DNS values and sync them with other services."
    )
    parser.add_argument("-c", "--config", type=str, help="load specified configuration file")
    args = parser.parse_args()

    # Did the user supply a config file?
    if args.config is None:
        config = NCMonitorConfig()
    else:
        config = NCMonitorConfig(config)

    domains = defaultdict(set)

    with ACMEChiefConfig() as acme:
        for domain in acme.get_domain_list():
            domains['acme_ready'].add(domain.domain_name)

    with NCRedirRedirects() as ncredir:
        for domain in ncredir.get_domain_list():
            domains['ncredir_ready'].add(domain.domain_name)

    domains_missing_redirects = domains['acme_ready'] - domains['ncredir_ready']
    domains_with_redirects = domains['acme_ready'] & domains['ncredir_ready']
    print(f"[*] domains missing redirects rules: {domains_missing_redirects}")
    print(f"[*] domains with redirects rules: {domains_with_redirects}")
    with MarkMonitor(config.mark_user, config.mark_pass) as mark:
        for domain in mark.get_domain_list():
            if domain.nameservers == config.nameservers:
                domains['mark_ready'].add(domain.domain_name)
            else:
                domains['mark_dns'].add(domain.domain_name)

    with DNSRepository() as dnsrepo:
        for domain in dnsrepo.get_domain_list():
            domains['dns_ready'].add(domain.domain_name)

    num_matching = len(domains['mark_ready'] & domains['dns_ready'])
    num_missing_ncredir = len(domains['mark_ready'] - domains['dns_ready'])
    num_missing_mark = len(domains['dns_ready'] - domains['mark_ready'])
    print(f"[*] {len(domains['mark_ready'])} domains in MarkMonitor with proper DNS servers")
    print(f"[*] {len(domains['mark_dns'])} domains in MarkMonitor with missing/wrong DNS servers")
    print(f"[*] {num_matching} domains in MarkMonitor && ncredir DNS zone")
    print(f"[*] {num_missing_ncredir} domains in MarkMonitor and ncredir DNS zone missing")
    print(f"[*] {num_missing_mark} domains with ncredir DNS zone missing from MarkMonitor")


if __name__ == '__main__':
    main()
